﻿Type=Service
Version=3
@EndOfDesignText@
#Region Module Attributes
	#StartAtBoot: False
#End Region

'Service module
Sub Process_Globals
	'These global variables will be declared once when the application starts.
	'These variables can be accessed from all modules.
Dim LastTX,LastRX As String
Dim Admin As BluetoothAdmin
Dim BT As Serial
Dim BTThread As Thread
Dim BTIn As InputStream
Dim BTOut As OutputStream
Dim Notification1 As Notification
Dim Connected As Boolean
Dim LastConnected As String
Dim PairedDevices As Map
Dim CurrentActivity As Object
Dim Debug As Boolean
Dim AdminEnabled As Boolean
Dim ServiceStarted As Boolean : ServiceStarted = False
Dim Sending As Boolean
End Sub

'External events
'OBD_NewData(Input() As String) array of obd bytes
'OBD_ServiceStarted
'0=Connect Fail 1=Enabling Admin 2=Admin Enabled 3=Connecting 4=Connected
'5=Admin Disabled 6=Disconnected 7=Devices ready
'OBD_ConnectStatus(Status As Int)
'External subs
'SetActivity(Component As Object) sets the global current activity
'EnableAdmin
'DisableAdmin
'GetDevices sets global PairedDevices
'Connect(Device As String)
'DisConnect
'Send(Str As String)
Sub Service_Create
Debug = True
Admin.Initialize("admin")
BT.Initialize("BT")
PairedDevices.Initialize
BTThread.Initialise("BTThread")
Notification1.Initialize 'dummy notification for connect
Connected = False
Sending = False
If Admin.IsEnabled = True Then
	AdminEnabled = True
Else
	AdminEnabled = False
End If
End Sub

Sub Service_Start(StartingIntent As Intent)
Service.StartForeground(1, Notification1)
ServiceStarted = True
If SubExists(CurrentActivity,"OBD_ServiceStarted") = True Then
	CallSub(CurrentActivity,"OBD_ServiceStarted") ' notify caller of success
End If
If AdminEnabled = True Then
	GetDevices
	CallSub2(CurrentActivity,"OBD_ConnectStatus",7) ' Ready for connect
Else
	EnableAdmin
End If	
End Sub

'Enable bluetooth
Sub EnableAdmin
If AdminEnabled = False Then
	Admin.Enable
	CallSub2(CurrentActivity,"OBD_ConnectStatus",1) ' Enabling Admin
End If
End Sub

'Disable bluetooth
Sub DisableAdmin
If Admin.IsEnabled = True Then
	Admin.Disable
End If
End Sub

Sub admin_StateChanged(NewState As Int, OldState As Int)
If NewState = Admin.STATE_ON Then
	AdminEnabled = True
	GetDevices
	CallSub2(CurrentActivity,"OBD_ConnectStatus",2) ' Admin Enabled
End If
If NewState = Admin.STATE_OFF Then
	AdminEnabled = False
	CallSub2(CurrentActivity,"OBD_ConnectStatus",3) ' Admin Disabled
End If
End Sub

Sub GetDevices
PairedDevices = BT.GetPairedDevices
End Sub

'e.g. Connect("00:01:10:11:20:21",Main)
Sub Connect(Device As String)
LastConnected = Device
If Admin.IsEnabled = True Then
	If LastConnected.Length < 10 Then
		GetDevices
		CallSub2(CurrentActivity,"OBD_ConnectStatus",7) ' Devices ready
		Return
	Else
		BT.ConnectInsecure(Admin,Device,1)
		CallSub2(CurrentActivity,"OBD_ConnectStatus",3) ' Connecting
	End If
Else '0=Connect Fail 1=Enabling Admin 2=Admin Enabled 3=Connecting 4=Connected 5=Admin Disabled 6=Disconnected
	EnableAdmin
End If
End Sub

Sub BT_Connected(Success As Boolean)
Dim args(0) As Object
If Debug = True Then Log("Connected: " & Success)
If Success Then
	BTIn = BT.InputStream
	BTOut = BT.OutputStream
	Connected = True
	If BTThread.Running = False Then
		BTThread.Start(Me,"BTThreadSub", args)
	End If
	CallSub2(CurrentActivity,"OBD_ConnectStatus",4) ' Connected
	'LastConnected = connectedDevice.Mac
Else
	LastConnected = ""
	Connected = False
	Log(LastException.Message)
	CallSub2(CurrentActivity,"OBD_ConnectStatus",0) ' Connect Fail
End If
End Sub

'************ BT Thread subs
Sub BTThreadSub
Dim sb As StringBuilder
Dim EOI As Boolean
Dim I As Int
Dim ts As String
Dim Ex As ExceptionEx
Dim msg As String
Dim Params(1) As Object
Dim InStr As String
Dim l,ix As Int
Try
sb.Initialize
Do While Connected = True
	EOI = False
	If BTIn.BytesAvailable > 0 Then
		Dim Buf(61) As Byte
		BTIn.ReadBytes(Buf,0,60)
		For I = 0 To Buf.Length - 1
			If Buf(I) > 32 Then
				If Buf(I) = 62 Then ' Found end of input marker
					EOI = True
				End If
				sb.Append(Chr(Buf(I)))
			End If
		Next
	End If
	If EOI = True Then
		ts = sb.ToString
		sb.Initialize
		InStr = ts.Replace(">","")
		LastRX = InStr
		l = InStr.Length
		Sending = False
		If InStr.StartsWith("4") AND l Mod 2 = 0 Then 'if theres an even number of bytes and is elm data
			Dim HexStr(l/2) As String
			I = 0 'put each pair of bytes into an array
			ix = 0
			Do While I < l
				HexStr(ix) = InStr.CharAt(I) & InStr.CharAt(I + 1)
				I = I + 2
				ix = ix + 1
			Loop
			Params(0) = HexStr ' each byte pair is in params
			BTThread.RunOnGuiThread("NewInput", Params)
		Else ' if not obd data then flag info. result is in lastrx
			Dim HexStr(1)
			HexStr(0) = "INFO"
			Params(0) = HexStr
			BTThread.RunOnGuiThread("NewInput", Params)
		End If
	End If
	BTThread.Sleep(50)
Loop
Catch
	Ex = LastException
	msg = Ex.StackTraceElement(0)
	Log(Ex.Name & " " & msg)
	Ex.Initialize("Error in Thread")
	Ex.Throw
End Try
End Sub

Sub NewInput(Input() As String)
If IsPaused(CurrentActivity) = False Then
	CallSub2(CurrentActivity,"OBD_NewData",Input)
End If
End Sub

' Sends a single command. Adds Chr(13)
Sub Send(Str As String)
Dim Buf() As Byte
Str = Str & Chr(13)
Buf = Str.GetBytes("UTF8")
LastTX = Str
Sending = True
Try
	BTOut.WriteBytes(Buf,0,Buf.Length)
Catch
	Log(LastException.Message)
	DisConnect
	ToastMessageShow("Cnnection Error", False)
End Try
End Sub

Sub DisConnect
If Connected Then
	Connected = False
'	BTIn.Close
'	BTOut.Close
'	BT.DisConnect
'	BTThread.Interrupt
End If
End Sub

Sub BTThread_Ended(fail As Boolean, error As String) 'An error or Exception has occurred in the Thread
If fail Then
	If Debug = True Then Log("Thread ended " & error)
Else
	If Debug = True Then Log("Thread ended")
End If
Connected = False
BTIn.Close
BTOut.Close
BT.DisConnect
If IsPaused(CurrentActivity) = False Then
	CallSub(CurrentActivity,"OBD_Disconnected")
End If
End Sub

Sub Stop
If Debug = True Then Log("OBDService finished")
'It will do nothing if the activity is currently paused.
'CallSub(CurrentActivity, "OBDStop")
Service.StopForeground(1) 'Return the service to the "background" (also removes the ongoing notification)
'If IsPaused(Main) Then        'The activity is paused. The user is probably busy with some other activity.
'	Notification1.Sound = True 'Notify the user that the download has finished
'	Notification1.SetInfo("Digidash", "OBDService Stop", Main)
'	Notification1.AutoCancel = True
'	Notification1.Notify(1)
'End If
End Sub

Sub Service_Destroy
ServiceStarted = False
End Sub

Sub tLog(Text As String,IncludeDateTime As Boolean)
Dim Str As String
If IncludeDateTime = True Then
	Str = DateTime.Date(DateTime.Now) & " "
	Str = Str & DateTime.Time(DateTime.Now) & " "
End If
Log(Str & Text)
End Sub
